"""Base module for logging in the Lion framework."""

from pydantic import Field, field_serializer

from lion_core.abc import ImmutableRecord
from lion_core.generic.element import Element
from lion_core.generic.note import Note
from lion_core.libs import to_dict


class BaseLog(Element, ImmutableRecord):
    content: Note = Field(
        default_factory=Note,
        title="Log Content",
        description="The content of the log entry.",
    )

    loginfo: Note = Field(
        default_factory=Note,
        title="Log Info",
        description="Metadata about the log entry.",
    )

    def __init__(self, content: Note, loginfo: Note, **kwargs):
        super().__init__(**kwargs)
        self.content = self._validate_note(content)
        self.loginfo = self._validate_note(loginfo)

    @classmethod
    def from_dict(cls, data: dict):
        return cls(
            content=cls._validate_note(data.get("content", {})),
            loginfo=cls._validate_note(data.get("loginfo", {})),
        )

    def _validate_note(cls, value):
        if not value:
            return Note()
        if isinstance(value, Note):
            return value
        if isinstance(value, dict):
            return Note(**value)
        try:
            return Note(**to_dict(value))
        except Exception as e:
            raise e

    @field_serializer("content", "loginfo")
    def _serialize_note(self, value):
        return value.to_dict()

    def to_dict(self):
        return {
            "content": self.content.to_dict(),
            "loginfo": self.loginfo.to_dict(),
        }

    def to_note(self):
        return Note(**self.to_dict())


__all__ = ["BaseLog"]
# File: lion_core/generic/log.py



from lion_core.session.msg_handlers.create_msg import create_message
from lion_core.session.msg_handlers.system_msg import validate_system
from lion_core.session.msg_handlers.validate_msg import validate_message

__all__ = [
    "validate_system",
    "create_message",
    "validate_message",
]



from typing import Type

from pydantic import Field, PrivateAttr, model_validator

from lion_core.abc import AbstractSpace
from lion_core.communication.system import System
from lion_core.generic.node import Node
from lion_core.generic.pile import Pile
from lion_core.imodel.imodel import iModel
from lion_core.session.msg_handlers import validate_system
from lion_core.sys_utils import SysUtil


class BaseSession(Node, AbstractSpace):
    system: System | None = Field(None)
    user: str | None = Field(None)
    imodel: iModel | None = Field(None)
    name: str | None = Field(None)
    pile_type: Type[Pile] = PrivateAttr(Pile)

    @model_validator(mode="before")
    def validate_system(cls, data: dict):
        system = data.pop("system", None)
        sender = data.pop("system_sender", None)
        system_datetime = data.pop("system_datetime", None)

        system = validate_system(
            system=system,
            sender=sender,
            system_datetime=system_datetime,
        )
        data["system"] = system
        return data

    @model_validator(mode="after")
    def check_system_recipient(self):
        if not SysUtil.is_id(self.system.recipient):
            self.system.recipient = self.ln_id
        return self


# File: lion_core/session/base.py



from setuptools import find_packages, setup

# Read the contents of the README file
with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

# Read the contents of the requirements file
with open("requirements.txt", "r", encoding="utf-8") as fh:
    requirements = fh.read().splitlines()

setup(
    name="lion_core",
    version="0.1.1",
    author="HaiyangLi",
    author_email="quantocean.li@gmail.com",
    description="A core package for the Lion framework.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/lion-agi/lion-core",
    packages=find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.11",
    install_requires=requirements,
    include_package_data=True,
    package_data={
        "": ["LICENSE", "README.md"],
    },
)



from typing import Any, Callable


def nfilter(
    nested_structure: dict[Any, Any] | list[Any], condition: Callable[[Any], bool]
) -> dict[Any, Any] | list[Any]:
    """
    Filter elements in a nested structure (dict or list) based on a condition.

    Args:
        nested_structure: The nested structure to filter.
        condition: A function that returns True for elements to keep and
            False for elements to discard.

    Returns:
        The filtered nested structure.

    Raises:
        TypeError: If nested_structure is not a dict or list.
    """
    if isinstance(nested_structure, dict):
        return _filter_dict(nested_structure, condition)
    elif isinstance(nested_structure, list):
        return _filter_list(nested_structure, condition)
    else:
        raise TypeError("The nested_structure must be either a dict or a list.")


def _filter_dict(
    dictionary: dict[Any, Any], condition: Callable[[tuple[Any, Any]], bool]
) -> dict[Any, Any]:
    """
    Filter elements in a dictionary based on a condition.

    Args:
        dictionary: The dictionary to filter.
        condition: A function that returns True for key-value pairs to keep
            and False for key-value pairs to discard.

    Returns:
        The filtered dictionary.
    """
    return {
        k: nfilter(v, condition) if isinstance(v, dict | list) else v
        for k, v in dictionary.items()
        if condition(v) or isinstance(v, dict | list)
    }


def _filter_list(lst: list[Any], condition: Callable[[Any], bool]) -> list[Any]:
    """
    Filter elements in a list based on a condition.

    Args:
        lst: The list to filter.
        condition: A function that returns True for elements to keep and
            False for elements to discard.

    Returns:
        The filtered list.
    """
    return [
        nfilter(item, condition) if isinstance(item, dict | list) else item
        for item in lst
        if condition(item) or isinstance(item, dict | list)
    ]


# Path: lion_core/libs/data_handlers/_nfilter.py
